/* Copyright 2018 The TensorFlow Authors. All Rights Reserved.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
=======================================================================*/

package org.tensorflow.op.core;

import javax.annotation.Generated;
import org.tensorflow.DataType;
import org.tensorflow.Operand;
import org.tensorflow.Operation;
import org.tensorflow.OperationBuilder;
import org.tensorflow.Output;
import org.tensorflow.op.PrimitiveOp;
import org.tensorflow.op.Scope;
import org.tensorflow.op.annotation.Operator;

/**
 * Deserialize and concatenate {@code SparseTensors} from a serialized minibatch.
 * <p>
 * The input {@code serialized_sparse} must be a string matrix of shape {@code [N x 3]} where
 * {@code N} is the minibatch size and the rows correspond to packed outputs of
 * {@code SerializeSparse}.  The ranks of the original {@code SparseTensor} objects
 * must all match.  When the final {@code SparseTensor} is created, it has rank one
 * higher than the ranks of the incoming {@code SparseTensor} objects
 * (they have been concatenated along a new row dimension).
 * <p>
 * The output {@code SparseTensor} object's shape values for all dimensions but the
 * first are the max across the input {@code SparseTensor} objects' shape values
 * for the corresponding dimensions.  Its first shape value is {@code N}, the minibatch
 * size.
 * <p>
 * The input {@code SparseTensor} objects' indices are assumed ordered in
 * standard lexicographic order.  If this is not the case, after this
 * step run {@code SparseReorder} to restore index ordering.
 * <p>
 * For example, if the serialized input is a {@code [2 x 3]} matrix representing two
 * original {@code SparseTensor} objects:
 * <p>
 *     index = [ 0]
 *             [10]
 *             [20]
 *     values = [1, 2, 3]
 *     shape = [50]
 * <p>
 * and
 * <p>
 *     index = [ 2]
 *             [10]
 *     values = [4, 5]
 *     shape = [30]
 * <p>
 * then the final deserialized {@code SparseTensor} will be:
 * <p>
 *     index = [0  0]
 *             [0 10]
 *             [0 20]
 *             [1  2]
 *             [1 10]
 *     values = [1, 2, 3, 4, 5]
 *     shape = [2 50]
 * 
 * @param <T> data type for {@code sparseValues()} output
 */
@Generated(value = "TensorFlow Java Op Generator")
@Operator(group = "core")
public final class DeserializeManySparse<T> extends PrimitiveOp {
  
  /**
   * Factory method to create a class to wrap a new DeserializeManySparse operation to the graph.
   * 
   * @param scope current graph scope
   * @param serializedSparse 2-D, The {@code N} serialized {@code SparseTensor} objects.
   * Must have 3 columns.
   * @param dtype The {@code dtype} of the serialized {@code SparseTensor} objects.
   * @return a new instance of DeserializeManySparse
   */
  public static <T> DeserializeManySparse<T> create(Scope scope, Operand<String> serializedSparse, Class<T> dtype) {
    OperationBuilder opBuilder = scope.graph().opBuilder("DeserializeManySparse", scope.makeOpName("DeserializeManySparse"));
    opBuilder.addInput(serializedSparse.asOutput());
    opBuilder.setAttr("dtype", DataType.fromClass(dtype));
    return new DeserializeManySparse<T>(opBuilder.build());
  }
  
  /**
   */
  public Output<Long> sparseIndices() {
    return sparseIndices;
  }
  
  /**
   */
  public Output<T> sparseValues() {
    return sparseValues;
  }
  
  /**
   */
  public Output<Long> sparseShape() {
    return sparseShape;
  }
  
  private Output<Long> sparseIndices;
  private Output<T> sparseValues;
  private Output<Long> sparseShape;
  
  private DeserializeManySparse(Operation operation) {
    super(operation);
    int outputIdx = 0;
    sparseIndices = operation.output(outputIdx++);
    sparseValues = operation.output(outputIdx++);
    sparseShape = operation.output(outputIdx++);
  }
}
