/* Copyright 2018 The TensorFlow Authors. All Rights Reserved.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
=======================================================================*/

package org.tensorflow.op.core;

import javax.annotation.Generated;
import org.tensorflow.Operand;
import org.tensorflow.Operation;
import org.tensorflow.OperationBuilder;
import org.tensorflow.Output;
import org.tensorflow.op.PrimitiveOp;
import org.tensorflow.op.Scope;
import org.tensorflow.op.annotation.Operator;

/**
 * Solves systems of linear equations with upper or lower triangular matrices by
 * <p>
 * backsubstitution.
 * <p>
 * {@code matrix} is a tensor of shape {@code [..., M, M]} whose inner-most 2 dimensions form
 * square matrices. If {@code lower} is {@code True} then the strictly upper triangular part
 * of each inner-most matrix is assumed to be zero and not accessed.
 * If {@code lower} is False then the strictly lower triangular part of each inner-most
 * matrix is assumed to be zero and not accessed.
 * {@code rhs} is a tensor of shape {@code [..., M, K]}.
 * <p>
 * The output is a tensor of shape {@code [..., M, K]}. If {@code adjoint} is
 * {@code True} then the innermost matrices in {@code output} satisfy matrix equations
 * {@code matrix[..., :, :] * output[..., :, :] = rhs[..., :, :]}.
 * If {@code adjoint} is {@code False} then the strictly then the  innermost matrices in
 * {@code output} satisfy matrix equations
 * {@code adjoint(matrix[..., i, k]) * output[..., k, j] = rhs[..., i, j]}.
 * 
 * @param <T> data type for {@code output()} output
 */
@Generated(value = "TensorFlow Java Op Generator")
@Operator(group = "core")
public final class MatrixTriangularSolve<T> extends PrimitiveOp implements Operand<T> {
  
  /**
   * Optional attributes for {@link org.tensorflow.op.core.MatrixTriangularSolve}
   */
  public static class Options {
    
    /**
     * @param lower Boolean indicating whether the innermost matrices in {@code matrix} are
     * lower or upper triangular.
     */
    public Options lower(Boolean lower) {
      this.lower = lower;
      return this;
    }
    
    /**
     * @param adjoint Boolean indicating whether to solve with {@code matrix} or its (block-wise)
     *          adjoint.
     * <p>
     * @compatibility(numpy)
     * Equivalent to scipy.linalg.solve_triangular
     * @end_compatibility
     */
    public Options adjoint(Boolean adjoint) {
      this.adjoint = adjoint;
      return this;
    }
    
    private Boolean lower;
    private Boolean adjoint;
    
    private Options() {
    }
  }
  
  /**
   * Factory method to create a class to wrap a new MatrixTriangularSolve operation to the graph.
   * 
   * @param scope current graph scope
   * @param matrix Shape is {@code [..., M, M]}.
   * @param rhs Shape is {@code [..., M, K]}.
   * @param options carries optional attributes values
   * @return a new instance of MatrixTriangularSolve
   */
  public static <T> MatrixTriangularSolve<T> create(Scope scope, Operand<T> matrix, Operand<T> rhs, Options... options) {
    OperationBuilder opBuilder = scope.graph().opBuilder("MatrixTriangularSolve", scope.makeOpName("MatrixTriangularSolve"));
    opBuilder.addInput(matrix.asOutput());
    opBuilder.addInput(rhs.asOutput());
    if (options != null) {
      for (Options opts : options) {
        if (opts.lower != null) {
          opBuilder.setAttr("lower", opts.lower);
        }
        if (opts.adjoint != null) {
          opBuilder.setAttr("adjoint", opts.adjoint);
        }
      }
    }
    return new MatrixTriangularSolve<T>(opBuilder.build());
  }
  
  /**
   * @param lower Boolean indicating whether the innermost matrices in {@code matrix} are
   * lower or upper triangular.
   */
  public static Options lower(Boolean lower) {
    return new Options().lower(lower);
  }
  
  /**
   * @param adjoint Boolean indicating whether to solve with {@code matrix} or its (block-wise)
   *          adjoint.
   * <p>
   * @compatibility(numpy)
   * Equivalent to scipy.linalg.solve_triangular
   * @end_compatibility
   */
  public static Options adjoint(Boolean adjoint) {
    return new Options().adjoint(adjoint);
  }
  
  /**
   * Shape is {@code [..., M, K]}.
   */
  public Output<T> output() {
    return output;
  }
  
  @Override
  public Output<T> asOutput() {
    return output;
  }
  
  private Output<T> output;
  
  private MatrixTriangularSolve(Operation operation) {
    super(operation);
    int outputIdx = 0;
    output = operation.output(outputIdx++);
  }
}
