/* Copyright 2018 The TensorFlow Authors. All Rights Reserved.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
=======================================================================*/

package org.tensorflow.op.core;

import javax.annotation.Generated;
import org.tensorflow.Operand;
import org.tensorflow.Operation;
import org.tensorflow.OperationBuilder;
import org.tensorflow.Output;
import org.tensorflow.op.PrimitiveOp;
import org.tensorflow.op.Scope;
import org.tensorflow.op.annotation.Operator;

/**
 * Computes the QR decompositions of one or more matrices.
 * <p>
 * Computes the QR decomposition of each inner matrix in {@code tensor} such that
 * {@code tensor[..., :, :] = q[..., :, :] * r[..., :,:])}
 * <pre>{@code
 * # a is a tensor.
 * # q is a tensor of orthonormal matrices.
 * # r is a tensor of upper triangular matrices.
 * q, r = qr(a)
 * q_full, r_full = qr(a, full_matrices=True)
 * }</pre>
 * 
 * 
 * @param <T> data type for {@code q()} output
 */
@Generated(value = "TensorFlow Java Op Generator")
@Operator(group = "core")
public final class Qr<T> extends PrimitiveOp {
  
  /**
   * Optional attributes for {@link org.tensorflow.op.core.Qr}
   */
  public static class Options {
    
    /**
     * @param fullMatrices If true, compute full-sized {@code q} and {@code r}. If false
     * (the default), compute only the leading {@code P} columns of {@code q}.
     */
    public Options fullMatrices(Boolean fullMatrices) {
      this.fullMatrices = fullMatrices;
      return this;
    }
    
    private Boolean fullMatrices;
    
    private Options() {
    }
  }
  
  /**
   * Factory method to create a class to wrap a new Qr operation to the graph.
   * 
   * @param scope current graph scope
   * @param input A tensor of shape {@code [..., M, N]} whose inner-most 2 dimensions
   * form matrices of size {@code [M, N]}. Let {@code P} be the minimum of {@code M} and {@code N}.
   * @param options carries optional attributes values
   * @return a new instance of Qr
   */
  public static <T> Qr<T> create(Scope scope, Operand<T> input, Options... options) {
    OperationBuilder opBuilder = scope.graph().opBuilder("Qr", scope.makeOpName("Qr"));
    opBuilder.addInput(input.asOutput());
    if (options != null) {
      for (Options opts : options) {
        if (opts.fullMatrices != null) {
          opBuilder.setAttr("full_matrices", opts.fullMatrices);
        }
      }
    }
    return new Qr<T>(opBuilder.build());
  }
  
  /**
   * @param fullMatrices If true, compute full-sized {@code q} and {@code r}. If false
   * (the default), compute only the leading {@code P} columns of {@code q}.
   */
  public static Options fullMatrices(Boolean fullMatrices) {
    return new Options().fullMatrices(fullMatrices);
  }
  
  /**
   * Orthonormal basis for range of {@code a}. If {@code full_matrices} is {@code False} then
   * shape is {@code [..., M, P]}; if {@code full_matrices} is {@code True} then shape is
   * {@code [..., M, M]}.
   */
  public Output<T> q() {
    return q;
  }
  
  /**
   * Triangular factor. If {@code full_matrices} is {@code False} then shape is
   * {@code [..., P, N]}. If {@code full_matrices} is {@code True} then shape is {@code [..., M, N]}.
   */
  public Output<T> r() {
    return r;
  }
  
  private Output<T> q;
  private Output<T> r;
  
  private Qr(Operation operation) {
    super(operation);
    int outputIdx = 0;
    q = operation.output(outputIdx++);
    r = operation.output(outputIdx++);
  }
}
