/* Copyright 2018 The TensorFlow Authors. All Rights Reserved.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
=======================================================================*/

package org.tensorflow.op.core;

import javax.annotation.Generated;
import org.tensorflow.DataType;
import org.tensorflow.Operand;
import org.tensorflow.Operation;
import org.tensorflow.OperationBuilder;
import org.tensorflow.Output;
import org.tensorflow.op.PrimitiveOp;
import org.tensorflow.op.Scope;
import org.tensorflow.op.annotation.Operator;

/**
 * Outputs random values from the Poisson distribution(s) described by rate.
 * <p>
 * This op uses two algorithms, depending on rate. If rate >= 10, then
 * the algorithm by Hormann is used to acquire samples via
 * transformation-rejection.
 * See http://www.sciencedirect.com/science/article/pii/0167668793909974.
 * <p>
 * Otherwise, Knuth's algorithm is used to acquire samples via multiplying uniform
 * random variables.
 * See Donald E. Knuth (1969). Seminumerical Algorithms. The Art of Computer
 * Programming, Volume 2. Addison Wesley
 * 
 * @param <V> data type for {@code output()} output
 */
@Generated(value = "TensorFlow Java Op Generator")
@Operator(group = "core")
public final class RandomPoissonV2<V extends Number> extends PrimitiveOp implements Operand<V> {
  
  /**
   * Optional attributes for {@link org.tensorflow.op.core.RandomPoissonV2}
   */
  public static class Options {
    
    /**
     * @param seed If either {@code seed} or {@code seed2} are set to be non-zero, the random number
     * generator is seeded by the given seed.  Otherwise, it is seeded by a
     * random seed.
     */
    public Options seed(Long seed) {
      this.seed = seed;
      return this;
    }
    
    /**
     * @param seed2 A second seed to avoid seed collision.
     */
    public Options seed2(Long seed2) {
      this.seed2 = seed2;
      return this;
    }
    
    private Long seed;
    private Long seed2;
    
    private Options() {
    }
  }
  
  /**
   * Factory method to create a class to wrap a new RandomPoissonV2 operation to the graph.
   * 
   * @param scope current graph scope
   * @param shape 1-D integer tensor. Shape of independent samples to draw from each
   * distribution described by the shape parameters given in rate.
   * @param rate A tensor in which each scalar is a "rate" parameter describing the
   * associated poisson distribution.
   * @param dtype 
   * @param options carries optional attributes values
   * @return a new instance of RandomPoissonV2
   */
  public static <V extends Number, T extends Number, U extends Number> RandomPoissonV2<V> create(Scope scope, Operand<T> shape, Operand<U> rate, Class<V> dtype, Options... options) {
    OperationBuilder opBuilder = scope.graph().opBuilder("RandomPoissonV2", scope.makeOpName("RandomPoissonV2"));
    opBuilder.addInput(shape.asOutput());
    opBuilder.addInput(rate.asOutput());
    opBuilder.setAttr("dtype", DataType.fromClass(dtype));
    if (options != null) {
      for (Options opts : options) {
        if (opts.seed != null) {
          opBuilder.setAttr("seed", opts.seed);
        }
        if (opts.seed2 != null) {
          opBuilder.setAttr("seed2", opts.seed2);
        }
      }
    }
    return new RandomPoissonV2<V>(opBuilder.build());
  }
  
  /**
   * @param seed If either {@code seed} or {@code seed2} are set to be non-zero, the random number
   * generator is seeded by the given seed.  Otherwise, it is seeded by a
   * random seed.
   */
  public static Options seed(Long seed) {
    return new Options().seed(seed);
  }
  
  /**
   * @param seed2 A second seed to avoid seed collision.
   */
  public static Options seed2(Long seed2) {
    return new Options().seed2(seed2);
  }
  
  /**
   * A tensor with shape {@code shape + shape(rate)}. Each slice
   * {@code [:, ..., :, i0, i1, ...iN]} contains the samples drawn for
   * {@code rate[i0, i1, ...iN]}.
   */
  public Output<V> output() {
    return output;
  }
  
  @Override
  public Output<V> asOutput() {
    return output;
  }
  
  private Output<V> output;
  
  private RandomPoissonV2(Operation operation) {
    super(operation);
    int outputIdx = 0;
    output = operation.output(outputIdx++);
  }
}
