/* Copyright 2018 The TensorFlow Authors. All Rights Reserved.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
=======================================================================*/

package org.tensorflow.op.core;

import javax.annotation.Generated;
import org.tensorflow.Operand;
import org.tensorflow.Operation;
import org.tensorflow.OperationBuilder;
import org.tensorflow.Output;
import org.tensorflow.op.PrimitiveOp;
import org.tensorflow.op.Scope;
import org.tensorflow.op.annotation.Operator;

/**
 * Applies sparse addition to {@code input} using individual values or slices
 * <p>
 * from {@code updates} according to indices {@code indices}.  The updates are non-aliasing:
 * {@code input} is only modified in-place if no other operations will use it.
 * Otherwise, a copy of {@code input} is made.  This operation has a gradient with
 * respect to both {@code input} and {@code updates}.
 * <p>
 * {@code input} is a {@code Tensor} with rank {@code P} and {@code indices} is a {@code Tensor} of rank {@code Q}.
 * <p>
 * {@code indices} must be integer tensor, containing indices into {@code input}.
 * It must be shape {@code [d_0, ..., d_{Q-2}, K]} where {@code 0 < K <= P}.
 * <p>
 * The innermost dimension of {@code indices} (with length {@code K}) corresponds to
 * indices into elements (if {@code K = P}) or {@code (P-K)}-dimensional slices
 * (if {@code K < P}) along the {@code K}th dimension of {@code input}.
 * <p>
 * {@code updates} is {@code Tensor} of rank {@code Q-1+P-K} with shape:
 * <pre>{@code
 * [d_0, ..., d_{Q-2}, input.shape[K], ..., input.shape[P-1]].
 * }</pre>
 * For example, say we want to add 4 scattered elements to a rank-1 tensor to 8
 * elements. In Python, that addition would look like this:
 * <p>
 *     input = tf.constant([1, 2, 3, 4, 5, 6, 7, 8])
 *     indices = tf.constant([[4], [3], [1], [7]])
 *     updates = tf.constant([9, 10, 11, 12])
 *     output = tf.scatter_nd_non_aliasing_add(input, indices, updates)
 *     with tf.Session() as sess:
 *       print(sess.run(output))
 * <p>
 * The resulting value {@code output} would look like this:
 * <p>
 *     [1, 13, 3, 14, 14, 6, 7, 20]
 * <p>
 * See @{tf.scatter_nd} for more details about how to make updates to slices.
 * 
 * @param <T> data type for {@code output()} output
 */
@Generated(value = "TensorFlow Java Op Generator")
@Operator(group = "core")
public final class ScatterNdNonAliasingAdd<T> extends PrimitiveOp implements Operand<T> {
  
  /**
   * Factory method to create a class to wrap a new ScatterNdNonAliasingAdd operation to the graph.
   * 
   * @param scope current graph scope
   * @param input A Tensor.
   * @param indices A Tensor. Must be one of the following types: {@code int32}, {@code int64}.
   * A tensor of indices into {@code input}.
   * @param updates A Tensor. Must have the same type as ref. A tensor of updated values
   * to add to {@code input}.
   * @return a new instance of ScatterNdNonAliasingAdd
   */
  public static <T, U extends Number> ScatterNdNonAliasingAdd<T> create(Scope scope, Operand<T> input, Operand<U> indices, Operand<T> updates) {
    OperationBuilder opBuilder = scope.graph().opBuilder("ScatterNdNonAliasingAdd", scope.makeOpName("ScatterNdNonAliasingAdd"));
    opBuilder.addInput(input.asOutput());
    opBuilder.addInput(indices.asOutput());
    opBuilder.addInput(updates.asOutput());
    return new ScatterNdNonAliasingAdd<T>(opBuilder.build());
  }
  
  /**
   * A {@code Tensor} with the same shape as {@code input}, containing values of {@code input}
   * updated with {@code updates}.
   */
  public Output<T> output() {
    return output;
  }
  
  @Override
  public Output<T> asOutput() {
    return output;
  }
  
  private Output<T> output;
  
  private ScatterNdNonAliasingAdd(Operation operation) {
    super(operation);
    int outputIdx = 0;
    output = operation.output(outputIdx++);
  }
}
