/* Copyright 2018 The TensorFlow Authors. All Rights Reserved.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
=======================================================================*/

package org.tensorflow.op.core;

import javax.annotation.Generated;
import org.tensorflow.Operand;
import org.tensorflow.Operation;
import org.tensorflow.OperationBuilder;
import org.tensorflow.Output;
import org.tensorflow.op.PrimitiveOp;
import org.tensorflow.op.Scope;
import org.tensorflow.op.annotation.Operator;

/**
 * Return substrings from {@code Tensor} of strings.
 * <p>
 * For each string in the input {@code Tensor}, creates a substring starting at index
 * {@code pos} with a total length of {@code len}.
 * <p>
 * If {@code len} defines a substring that would extend beyond the length of the input
 * string, then as many characters as possible are used.
 * <p>
 * If {@code pos} is negative or specifies a character index larger than any of the input
 * strings, then an {@code InvalidArgumentError} is thrown.
 * <p>
 * {@code pos} and {@code len} must have the same shape, otherwise a {@code ValueError} is thrown on
 * Op creation.
 * <p>
 * <i>NOTE</i>: {@code Substr} supports broadcasting up to two dimensions. More about
 * broadcasting
 * <a href="http://docs.scipy.org/doc/numpy/user/basics.broadcasting.html">here</a>
 * <p>
 * ---
 * <p>
 * Examples
 * <p>
 * Using scalar {@code pos} and {@code len}:
 * <pre>{@code
 * input = [b'Hello', b'World']
 * position = 1
 * length = 3
 * 
 * output = [b'ell', b'orl']
 * }</pre>
 * Using {@code pos} and {@code len} with same shape as {@code input}:
 * <pre>{@code
 * input = [[b'ten', b'eleven', b'twelve'],
 *          [b'thirteen', b'fourteen', b'fifteen'],
 *          [b'sixteen', b'seventeen', b'eighteen']]
 * position = [[1, 2, 3],
 *             [1, 2, 3],
 *             [1, 2, 3]]
 * length =   [[2, 3, 4],
 *             [4, 3, 2],
 *             [5, 5, 5]]
 * 
 * output = [[b'en', b'eve', b'lve'],
 *           [b'hirt', b'urt', b'te'],
 *           [b'ixtee', b'vente', b'hteen']]
 * }</pre>
 * Broadcasting {@code pos} and {@code len} onto {@code input}:
 * <pre>{@code
 * input = [[b'ten', b'eleven', b'twelve'],
 *          [b'thirteen', b'fourteen', b'fifteen'],
 *          [b'sixteen', b'seventeen', b'eighteen'],
 *          [b'nineteen', b'twenty', b'twentyone']]
 * position = [1, 2, 3]
 * length =   [1, 2, 3]
 * 
 * output = [[b'e', b'ev', b'lve'],
 *           [b'h', b'ur', b'tee'],
 *           [b'i', b've', b'hte'],
 *           [b'i', b'en', b'nty']]
 * }</pre>
 * Broadcasting {@code input} onto {@code pos} and {@code len}:
 * <pre>{@code
 * input = b'thirteen'
 * position = [1, 5, 7]
 * length =   [3, 2, 1]
 * 
 * output = [b'hir', b'ee', b'n']
 * }</pre>
 * 
 */
@Generated(value = "TensorFlow Java Op Generator")
@Operator(group = "core")
public final class Substr extends PrimitiveOp implements Operand<String> {
  
  /**
   * Factory method to create a class to wrap a new Substr operation to the graph.
   * 
   * @param scope current graph scope
   * @param input Tensor of strings
   * @param pos Scalar defining the position of first character in each substring
   * @param len Scalar defining the number of characters to include in each substring
   * @return a new instance of Substr
   */
  public static <T extends Number> Substr create(Scope scope, Operand<String> input, Operand<T> pos, Operand<T> len) {
    OperationBuilder opBuilder = scope.graph().opBuilder("Substr", scope.makeOpName("Substr"));
    opBuilder.addInput(input.asOutput());
    opBuilder.addInput(pos.asOutput());
    opBuilder.addInput(len.asOutput());
    return new Substr(opBuilder.build());
  }
  
  /**
   * Tensor of substrings
   */
  public Output<String> output() {
    return output;
  }
  
  @Override
  public Output<String> asOutput() {
    return output;
  }
  
  private Output<String> output;
  
  private Substr(Operation operation) {
    super(operation);
    int outputIdx = 0;
    output = operation.output(outputIdx++);
  }
}
