/* Copyright 2018 The TensorFlow Authors. All Rights Reserved.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
=======================================================================*/

package org.tensorflow.op.core;

import javax.annotation.Generated;
import org.tensorflow.Operand;
import org.tensorflow.Operation;
import org.tensorflow.OperationBuilder;
import org.tensorflow.Output;
import org.tensorflow.op.PrimitiveOp;
import org.tensorflow.op.Scope;
import org.tensorflow.op.annotation.Operator;

/**
 * Computes the singular value decompositions of one or more matrices.
 * <p>
 * Computes the SVD of each inner matrix in {@code input} such that
 * {@code input[..., :, :] = u[..., :, :] * diag(s[..., :, :]) * transpose(v[..., :, :])}
 * <pre>{@code
 * # a is a tensor containing a batch of matrices.
 * # s is a tensor of singular values for each matrix.
 * # u is the tensor containing of left singular vectors for each matrix.
 * # v is the tensor containing of right singular vectors for each matrix.
 * s, u, v = svd(a)
 * s, _, _ = svd(a, compute_uv=False)
 * }</pre>
 * 
 * 
 * @param <T> data type for {@code s()} output
 */
@Generated(value = "TensorFlow Java Op Generator")
@Operator(group = "core")
public final class Svd<T> extends PrimitiveOp {
  
  /**
   * Optional attributes for {@link org.tensorflow.op.core.Svd}
   */
  public static class Options {
    
    /**
     * @param computeUv If true, left and right singular vectors will be
     * computed and returned in {@code u} and {@code v}, respectively.
     * If false, {@code u} and {@code v} are not set and should never referenced.
     */
    public Options computeUv(Boolean computeUv) {
      this.computeUv = computeUv;
      return this;
    }
    
    /**
     * @param fullMatrices If true, compute full-sized {@code u} and {@code v}. If false
     * (the default), compute only the leading {@code P} singular vectors.
     * Ignored if {@code compute_uv} is {@code False}.
     */
    public Options fullMatrices(Boolean fullMatrices) {
      this.fullMatrices = fullMatrices;
      return this;
    }
    
    private Boolean computeUv;
    private Boolean fullMatrices;
    
    private Options() {
    }
  }
  
  /**
   * Factory method to create a class to wrap a new Svd operation to the graph.
   * 
   * @param scope current graph scope
   * @param input A tensor of shape {@code [..., M, N]} whose inner-most 2 dimensions
   * form matrices of size {@code [M, N]}. Let {@code P} be the minimum of {@code M} and {@code N}.
   * @param options carries optional attributes values
   * @return a new instance of Svd
   */
  public static <T> Svd<T> create(Scope scope, Operand<T> input, Options... options) {
    OperationBuilder opBuilder = scope.graph().opBuilder("Svd", scope.makeOpName("Svd"));
    opBuilder.addInput(input.asOutput());
    if (options != null) {
      for (Options opts : options) {
        if (opts.computeUv != null) {
          opBuilder.setAttr("compute_uv", opts.computeUv);
        }
        if (opts.fullMatrices != null) {
          opBuilder.setAttr("full_matrices", opts.fullMatrices);
        }
      }
    }
    return new Svd<T>(opBuilder.build());
  }
  
  /**
   * @param computeUv If true, left and right singular vectors will be
   * computed and returned in {@code u} and {@code v}, respectively.
   * If false, {@code u} and {@code v} are not set and should never referenced.
   */
  public static Options computeUv(Boolean computeUv) {
    return new Options().computeUv(computeUv);
  }
  
  /**
   * @param fullMatrices If true, compute full-sized {@code u} and {@code v}. If false
   * (the default), compute only the leading {@code P} singular vectors.
   * Ignored if {@code compute_uv} is {@code False}.
   */
  public static Options fullMatrices(Boolean fullMatrices) {
    return new Options().fullMatrices(fullMatrices);
  }
  
  /**
   * Singular values. Shape is {@code [..., P]}.
   */
  public Output<T> s() {
    return s;
  }
  
  /**
   * Left singular vectors. If {@code full_matrices} is {@code False} then shape is
   * {@code [..., M, P]}; if {@code full_matrices} is {@code True} then shape is
   * {@code [..., M, M]}. Undefined if {@code compute_uv} is {@code False}.
   */
  public Output<T> u() {
    return u;
  }
  
  /**
   * Left singular vectors. If {@code full_matrices} is {@code False} then shape is
   * {@code [..., N, P]}. If {@code full_matrices} is {@code True} then shape is {@code [..., N, N]}.
   * Undefined if {@code compute_uv} is false.
   */
  public Output<T> v() {
    return v;
  }
  
  private Output<T> s;
  private Output<T> u;
  private Output<T> v;
  
  private Svd(Operation operation) {
    super(operation);
    int outputIdx = 0;
    s = operation.output(outputIdx++);
    u = operation.output(outputIdx++);
    v = operation.output(outputIdx++);
  }
}
