/* Copyright 2018 The TensorFlow Authors. All Rights Reserved.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
=======================================================================*/

package org.tensorflow.op.core;

import javax.annotation.Generated;
import org.tensorflow.DataType;
import org.tensorflow.Operand;
import org.tensorflow.Operation;
import org.tensorflow.OperationBuilder;
import org.tensorflow.Output;
import org.tensorflow.op.PrimitiveOp;
import org.tensorflow.op.Scope;
import org.tensorflow.op.annotation.Operator;

/**
 * Finds unique elements along an axis of a tensor.
 * <p>
 * This operation either returns a tensor {@code y} containing unique elements
 * along the {@code axis} of a tensor. The returned unique elements is sorted
 * in the same order as they occur along {@code axis} in {@code x}.
 * This operation also returns a tensor {@code idx} that is the same size as
 * the number of the elements in {@code x} along the {@code axis} dimension. It
 * contains the index in the unique output {@code y}.
 * In other words, for an {@code 1-D} tensor {@code x} with {@code axis = None:
 * 
 * }y[idx[i]] = x[i] for i in [0, 1,...,rank(x) - 1]{@code 
 * 
 * For example:
 * 
 * }{@code 
 * # tensor 'x' is [1, 1, 2, 4, 4, 4, 7, 8, 8]
 * y, idx = unique(x)
 * y ==> [1, 2, 4, 7, 8]
 * idx ==> [0, 0, 1, 2, 2, 2, 3, 4, 4]
 * }{@code 
 * 
 * For an }2-D{@code  tensor }x{@code  with }axis = 0{@code :
 * 
 * }{@code 
 * # tensor 'x' is [[1, 0, 0],
 * #                [1, 0, 0],
 * #                [2, 0, 0]]
 * y, idx = unique(x, axis=0)
 * y ==> [[1, 0, 0],
 *        [2, 0, 0]]
 * idx ==> [0, 0, 1]
 * }{@code 
 * 
 * For an }2-D{@code  tensor }x{@code  with }axis = 1{@code :
 * 
 * }{@code 
 * # tensor 'x' is [[1, 0, 0],
 * #                [1, 0, 0],
 * #                [2, 0, 0]]
 * y, idx = unique(x, axis=1)
 * y ==> [[1, 0],
 *        [1, 0],
 *        [2, 0]]
 * idx ==> [0, 1, 1]
 * }`
 * 
 * @param <T> data type for {@code y()} output
 * @param <V> data type for {@code idx()} output
 */
@Generated(value = "TensorFlow Java Op Generator")
@Operator(group = "core")
public final class UniqueV2<T, V extends Number> extends PrimitiveOp {
  
  /**
   * Factory method to create a class to wrap a new UniqueV2 operation to the graph.
   * 
   * @param scope current graph scope
   * @param x A {@code Tensor}.
   * @param axis A {@code Tensor} of type {@code int32} (default: None). The axis of the Tensor to
   * find the unique elements.
   * @param outIdx 
   * @return a new instance of UniqueV2
   */
  public static <T, V extends Number, U extends Number> UniqueV2<T, V> create(Scope scope, Operand<T> x, Operand<U> axis, Class<V> outIdx) {
    OperationBuilder opBuilder = scope.graph().opBuilder("UniqueV2", scope.makeOpName("UniqueV2"));
    opBuilder.addInput(x.asOutput());
    opBuilder.addInput(axis.asOutput());
    opBuilder.setAttr("out_idx", DataType.fromClass(outIdx));
    return new UniqueV2<T, V>(opBuilder.build());
  }
  
  /**
   * A {@code Tensor}. Unique elements along the {@code axis} of {@code Tensor} x.
   */
  public Output<T> y() {
    return y;
  }
  
  /**
   * A 1-D Tensor. Has the same type as x that contains the index of each
   * value of x in the output y.
   */
  public Output<V> idx() {
    return idx;
  }
  
  private Output<T> y;
  private Output<V> idx;
  
  private UniqueV2(Operation operation) {
    super(operation);
    int outputIdx = 0;
    y = operation.output(outputIdx++);
    idx = operation.output(outputIdx++);
  }
}
